/**********************************************************
 Project		: emit_v11
 Files			: emit_v11.c, lcd.h, lcd.c
 Description	: Universal shutterglasses controller
 
 Target	: Atmega16
 frequ	: 16MHz
 
 Modification date	: 21/05/2010
**********************************************************/
// Files :
#include <stdlib.h>
#include <stdio.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include "lcd.h"

// Equivalences :
#define BIT0 0b00000001
#define BIT1 0b00000010
#define BIT2 0b00000100
#define BIT3 0b00001000
#define BIT4 0b00010000
#define BIT5 0b00100000
#define BIT6 0b01000000
#define BIT7 0b10000000

//Nvidia IR pulses timings :
#define nv_lon_1h 	47	//23,5s
#define nv_lon_1l 	42	//21s
#define nv_lon_2h 	47	//23,5s

#define nv_ron_1h 	47	//23,5s
#define nv_ron_1l 	156	//78s
#define nv_ron_2h 	80	//40s

#define nv_loff_1h 	87	//43,5s

#define nv_roff_1h 	47	//23,5s
#define nv_roff_1l 	92	//46s
#define nv_roff_2h 	62	//31s

#define nv_off_1h 	136	//68s
#define nv_off_1l 	900	//450s
#define nv_off_2h 	136	//68s

#define nv_nbrpuls_l 	4
#define nv_nbrpuls_r 	3
#define nv_nbrpuls_off 	2
#define nv_ddc_ir_ddc_delay 10
#define nv_rx_timeout 	255	//127s

//Xpand IR pulses timings :
#define xp_0h 		30	//15s
#define xp_0l 		138	//69s

#define xp_1h 		30	//15s
#define xp_1l 		54	//27s

#define xp_nbrpuls_l 2
#define xp_nbrpuls_r 3

#define xp_code 	0

//Crystaleyes IR pulses timings
#define cr_lh 		120	//60s
#define cr_rh 		240	//120s
#define cr_nbrpuls_l 1
#define cr_nbrpuls_r 1

//IR_OFF parameters:
#define ir_off_delay 	0
#define ir_off_invert 	0

//Nvidia parameters :
#define nv_delay 		8900
#define nv_invert 		0
#define nv_darktime 	2600

//Nvidia+DDC parameters :
#define nv_ddc_delay 	11330
#define nv_ddc_invert 	0
#define nv_ddc_darktime 740

//Xpand parameters :
#define xp_delay 	0
#define xp_invert 	1

//Crystaleyes parameters :
#define cr_delay 	0
#define cr_invert 	0

//Other definitions
#define ir_off 		0
#define nvidia 		1
#define nvidia_ddc 	2
#define xpand 		3
#define crystaleyes 4

#define left 		0
#define right 		1
#define off 		2

//Function prototypes :
static int lcd_putchar(char c, FILE *stream);
static int uart_putchar(char c);
static FILE uart = FDEV_SETUP_STREAM(uart_putchar, NULL, _FDEV_SETUP_WRITE);
static FILE lcd = FDEV_SETUP_STREAM(lcd_putchar, NULL, _FDEV_SETUP_WRITE);
void ir_out(void);
void ddc_out(void);
void load_pulses(void);
void set_parameters(void);
unsigned int aquis_rot(unsigned int, unsigned int, unsigned int, unsigned int);
char push_rot(void);
char push_bt1(void);
char push_bt2(void);

// Input definitions
#define ir_rx	((PIND & BIT3) >> 3)
#define rot1	((PINA & BIT3) >> 3)
#define rot2	((PINA & BIT4) >> 4)
#define bt_rot	((PINA & BIT5) >> 5)
#define bt1 	((PINA & BIT6) >> 6)
#define bt2 	((PINA & BIT7) >> 7)

//input/output mapping :

//PA0 : out DDC
//PA1 : NC
//PA2 : NC
//PA3 : in	rot1
//PA4 : in	rot2
//PA5 : in	bt_rot
//PA6 : in	bt1
//PA7 : in	bt2

//PB0 : out	LED sync
//PB1 : NC
//PB2 : in	Vsync (INT2)
//PB3 : NC
//PB4 : NC
//PB5 : ISP MOSI
//PB6 : ISP MISO
//PB7 : ISP SCK

//PC0 : backlight
//PC1 : out LCD_bit7
//PC2 : out LCD_bit6
//PC3 : out LCD_bit5
//PC4 : out LCD_bit4
//PC5 : out LCD_E
//PC6 : out LCD_R/W
//PC7 : out LCD_RS

//PD0 : in  RX
//PD1 : out TX
//PD2 : in	IR_remote (INT0)
//PD3 : in	IR_sync (INT1)
//PD4 : out color corrector ON/OFF
//PD5 : out color corrector mode
//PD6 : NC
//PD7 : out IR_out

//Buttons variables
volatile unsigned char	buff_rot1;
volatile unsigned char	buff_rot2;
volatile unsigned char	buff_bt_rot;
volatile unsigned char	buff_bt1;
volatile unsigned char	buff_bt2;

//synchro
volatile unsigned char	rx_sync;
volatile unsigned char	out_sync;
volatile unsigned char	ok_sync;

//ir_rx
volatile unsigned char 	cpt_pulses_rx;
volatile unsigned char 	flag_rx;
volatile unsigned char	flag_ir_rx;
volatile unsigned char	inhib_rx;

//ir_tx
volatile unsigned int	ir_tx[96];
volatile unsigned char	no_pulse;
volatile unsigned char	nbr_pulses;

//ddc_out
volatile unsigned int	ir_ddc_delay;
volatile unsigned char	int_ddc;

//code Xpand X01
volatile unsigned char	no_bit;
volatile unsigned char	i;
volatile unsigned char	send_code;
volatile unsigned char	div_code;

//Other variables :
volatile unsigned char	init=0;
volatile unsigned char	ok_ir_tx;
volatile unsigned char	ok_delay;
volatile unsigned char	screen;
volatile unsigned char	mode=0;
volatile unsigned char	mode_p;
volatile unsigned char	shutter_model;
volatile unsigned int	cons_delay;
volatile unsigned int	cons_darktime;
volatile unsigned int	darktime;
volatile unsigned int	cons_invert;
volatile unsigned char	cons_color_correct;
volatile unsigned char	ddc_enable;
volatile unsigned char	ir_enable;
volatile unsigned char	mode_test;


// ==============================   ZONE  PROGRAMME ==============================

int main(void)
{	
	//Config portA
	DDRA=0b00000001;				//PA0 : out (DDC out)			
	PORTA=0x00;
	
	//Config portB
	DDRB=0b00000001;				//PB0 : out (sync led)			
	PORTB=0x00;						//(sync led : off)

	//Config portC
	DDRC |= 0b00000001;				//PC0 : out (backlight)			
	PORTC &= ~BIT0;					//backlight off
	
	//Config portD
	DDRD=0b10110010;				//PD1, PD4, PD5, PD7 : out
	PORTD=0x00;						
	
	//Config Timer0
	TCCR0=0b00000101;				//clk=fquartz/1024
	TCNT0=72;						//freq = 85Hz
	
	//config timer1
	TCCR1A=0b00000000;				//Normal mode
	TCCR1B=0b00000010;   			//clk timer1 = clk/8 Fmin = 62Hz
	
	//Config Timer2
	TCCR2=0b00000010;				//Normal mode, CLK=Fquartz/8
	
	//config UART
	UBRRL=207;	UBRRH=0;			//baud rate = 9600 bauds
	UCSRA=0b00000010;				//double baud rate
	UCSRB=0b00001000;				//Tx active
	UCSRC=0b10000110;				//Asynchronous, no parity, 1 stop bit, 8bit character
	
	//External interrupts
	MCUCR=0b00000101;				//INT1 on change, INT0 on change
	MCUCSR=0b01000000;				//INT2 on rising edge
	
	//Interrupts enable 
	GICR=0b11100000;				//Enable INT1, INT0 and INT2
	TIMSK=0b11011001;				//Enable interrupts : T2compare, T2OVF, T1compare 1A,T1compare 1B, T0OVF
	sei (); 						//Enable all interrupts

	//LCD init
    lcd_init(LCD_DISP_ON);
	
	//Serial out message
	fprintf_P(&uart, PSTR("test serial port\n"));			//Test message on serial port
	
	while (1)
	{
		switch (screen)
		{
			case 0:		//mode selection screen
				lcd_gotoxy(0,0);								//1st character, 1st line
				fprintf_P(&lcd, PSTR("Mode :          "));		//LCD write
				
				switch (mode)
				{
					case ir_off :
						lcd_gotoxy(0,1);							//1st character, 2nd line
						fprintf_P(&lcd, PSTR("IR OFF          "));	//LCD write
					break;
					
					case nvidia :
						lcd_gotoxy(0,1);							//1st character, 2nd line
						fprintf_P(&lcd, PSTR("3D Vision       "));	//LCD write
					break;
					
					case nvidia_ddc :
						lcd_gotoxy(0,1);							//1st character, 2nd line
						fprintf_P(&lcd, PSTR("3D Vision + DDC "));	//LCD write
					break;
					
					case xpand :
						lcd_gotoxy(0,1);							//1st character, 2nd line
						fprintf_P(&lcd, PSTR("Xpand X101      "));	//LCD write
					break;
					
					case crystaleyes :
						lcd_gotoxy(0,1);							//1st character, 2nd line
						fprintf_P(&lcd, PSTR("Crystaleyes     "));	//LCD write
					break;
				}
				
				mode = aquis_rot(mode, 1, 0, 4);
				
				if(mode != mode_p)
					set_parameters();
				
				mode_p = mode;
			break;
			
			case 1:		//color corrector screen
				lcd_gotoxy(0,0);							//1st character, 1st line
				fprintf_P(&lcd, PSTR("Color corrector:"));	//LCD write
				
				switch (cons_color_correct)
				{
					case 0 :
						lcd_gotoxy(0,1);							//1st character, 2nd line
						fprintf_P(&lcd, PSTR("OFF             "));	//LCD write
					break;
					
					case 1 :
						lcd_gotoxy(0,1);							//1st character, 2nd line
						fprintf_P(&lcd, PSTR("ON mode 1       "));	//LCD write
					break;
					
					case 2 :
						lcd_gotoxy(0,1);							//1st character, 2nd line
						fprintf_P(&lcd, PSTR("ON mode 2       "));	//LCD write
					break;
				}
				
				cons_color_correct = aquis_rot(cons_color_correct, 1, 0, 2);
			break;
			
			case 2:		//reverse 3D screen
				lcd_gotoxy(0,0);							//1st character, 1st line
				fprintf_P(&lcd, PSTR("Normal/Reverse :"));	//LCD write
				
				if(cons_invert)
				{
					lcd_gotoxy(0,1);							//1st character, 2nd line
					fprintf_P(&lcd, PSTR("Reverse         "));	//LCD write
				}
				else
				{
					lcd_gotoxy(0,1);							//1st character, 2nd line
					fprintf_P(&lcd, PSTR("Normal          "));	//LCD write
				}
				
				cons_invert=aquis_rot(cons_invert, 1, 0, 1);
			break;
			
			case 3:		//Delay screen
				lcd_gotoxy(0,0);							//1st character, 1st line
				fprintf_P(&lcd, PSTR("Delay :         "));	//LCD write
				
				lcd_gotoxy(0,1);										//1st character, 2nd line
				fprintf_P(&lcd, PSTR("%.5dus         "), cons_delay);	//LCD write
				
				cons_delay = aquis_rot(cons_delay, 20, 0, 20000);
			break;
			
			case 4:		//Dark time screen
				lcd_gotoxy(0,0);							//1st character, 1st line
				fprintf_P(&lcd, PSTR("Dark Time :     "));	//LCD write
				
				lcd_gotoxy(0,1);											//1st character, 2nd line
				fprintf_P(&lcd, PSTR("%.5dus         "), cons_darktime);	//LCD write
				
				cons_darktime = aquis_rot(cons_darktime, 20, 740, 10000);
				darktime = cons_darktime*2;
			break;
			
			case 5:		//Test screen
				lcd_gotoxy(0,0);							//1st character, 1st line
				fprintf_P(&lcd, PSTR("Test Mode :     "));	//LCD write
				
				if(mode_test)
				{
					lcd_gotoxy(0,1);							//1st character, 2nd line
					fprintf_P(&lcd, PSTR("ON              "));	//LCD write
				}
				else
				{
					lcd_gotoxy(0,1);							//1st character, 2nd line
					fprintf_P(&lcd, PSTR("OFF             "));	//LCD write
				}
				
				mode_test=aquis_rot(mode_test, 1, 0, 1);
			break;
		}
		
		//previous screen
		if(push_bt1())
		{
			if(screen > 0)
				screen--;
			else
				screen=5;
		}
		
		//next screen
		if(push_bt2())
		{
			if(screen < 5)
				screen++;
			else
				screen=0;
		}
		
		//shortcut
		if(push_rot())
		{
			//if collor corrector enabled : toggle mode
			if(cons_color_correct == 1)
				cons_color_correct = 2;
			
			else if(cons_color_correct == 2)
				cons_color_correct = 1;
			
			//if collor collector disabled : toggle Normal/invert 3D
			else if(cons_invert == 0)
				cons_invert=1;
			
			else
				cons_invert=0;
		}
		
		switch (cons_color_correct)
		{
			case 0 :	//OFF
				PORTD &= ~BIT4;
				PORTD &= ~BIT5;
			break;
			
			case 1 :	//Mode1
				PORTD |= BIT4;
				PORTD &= ~BIT5;
			break;
			
			case 2 :	//Mode2
				PORTD |= BIT4;
				PORTD |= BIT5;
			break;
		}
		
		if(init == 0)
		{
			set_parameters();
			load_pulses();
			init=1;
		}
	}
}

// Interrupt INT0 (remote control)
SIGNAL(SIG_INTERRUPT0)
{
}

// Interrupt INT1 (Nvidia IR pulses rx)
SIGNAL(SIG_INTERRUPT1)
{
	if (inhib_rx == 0 && init == 1)
	{
		if(ir_rx == 0)			//Falling edge (ir pulse)
		{
			if(flag_rx == 0)		//if 1st pulse
			{
				OCR2 = TCNT2 + nv_rx_timeout;		//set timeout rx
				flag_rx = 1;
			}
		}
		else
			cpt_pulses_rx++;
	}
}

// Interrupt INT2 (VGA Vsync pin)
SIGNAL(SIG_INTERRUPT2)
{
	if(cons_delay != 0)
		OCR1A = TCNT1 + (cons_delay*2);
		
	switch (rx_sync)
	{
		case left :
			if(cons_invert)
				out_sync = right;
			else
				out_sync = left;
		break;
		
		case right :
			if(cons_invert)
				out_sync = left;
			else
				out_sync = right;
		break;
		
		case off :
			out_sync = off;
		break;
	}
	
	if(flag_ir_rx)
	{
		flag_ir_rx = 0;
		
		if(cons_delay == 0)
		{
			if(ir_enable)
				ir_out();
			else if(ddc_enable)
				ddc_out();
		}
		else
			ok_delay=1;
	}
}

//Timer 0 overflow
SIGNAL(SIG_OVERFLOW0)
{
	if(init)
	{
		TCNT0=72;				//freq = 85Hz
		if(mode_test == 1)
		{
			if(out_sync)
			{
				out_sync = left;
				out_sync = left;
			}
			else
			{
				out_sync = right;
				out_sync = right;
			}
			
			if(ir_enable)
				ir_out();
			else if(ddc_enable)
				ddc_out();
		}
		
		//Xpand X101 code management
		if(shutter_model == 2)		//if Xpand X101
		{
			div_code++;
			if(div_code > 8)
			{
				div_code=0;
				send_code = 1;
			}
		}
	}
}

//compare Timer1A (Delay)
SIGNAL(SIG_OUTPUT_COMPARE1A)
{
	if(ok_delay)
	{
		ok_delay=0;
		if(ir_enable)
			ir_out();
		else if(ddc_enable)
			ddc_out();
	}
	
	if(ok_sync)
		ok_sync = 0;
	else
		PORTB &= ~BIT0;		//sync led OFF
}

//compare Timer1B (IR TX & DDC)
SIGNAL(SIG_OUTPUT_COMPARE1B)
{
	if(ok_ir_tx)
	{
		if(no_pulse & BIT0)
			PORTD |= BIT7;				//IR out = 1
		else
			PORTD &= ~BIT7;				//IR out = 0
		
		no_pulse++;
		OCR1B=TCNT1+ir_tx[no_pulse];	//update compare1B
		
		if(no_pulse == nbr_pulses)		//end of transmission
		{
			nbr_pulses=0;
			no_pulse=0;	
			ok_ir_tx=0;
			PORTD &= ~BIT7;				//IR out = 0
			
			if(ddc_enable)
			{
				int_ddc=1;
				OCR1B=TCNT1+ir_ddc_delay-6;
			}
			else
				PORTA &= ~BIT0;			//DDC out = 0
			
			load_pulses();
		}
	}
	
	else if (int_ddc)			//Update DDC signal
	{
		ddc_out();		
		int_ddc=0;
	}
}

//Timer 2 OVF
SIGNAL(SIG_OVERFLOW2)
{
}

//Interruption Timer2 compare
SIGNAL(SIG_OUTPUT_COMPARE2)
{
	if(flag_rx)				//nvidia pulses rx
	{
		if(cpt_pulses_rx > 1)			//if code = right_off
			rx_sync = left;
		else							//if code = left_off
		{
			if(rx_sync == right || rx_sync == off)
				rx_sync = off;
			else
				rx_sync= right;
		}
		
		flag_ir_rx = 1;
		ok_sync = 1;
		PORTB |= BIT0;					//sync led ON
		
		//init rx
		flag_rx = 0;
		cpt_pulses_rx = 0;
		
		//RX inhibition = 562s(4x128s+50s)
		inhib_rx=5;
		OCR2=TCNT2+100;
	}
	else if (inhib_rx > 0)
		inhib_rx--;
}

void ir_out(void)
{
	ok_ir_tx = 1;
	PORTD |= BIT7;				//IR out = 1
	OCR1B = TCNT1+ir_tx[0];		//update compare1B
}

void ddc_out(void)
{
	switch (out_sync)
	{
		case left :
			PORTA &= ~BIT0;				//DDC out = 0
		break;
		
		case right :
			PORTA |= BIT0;				//DDC out = 1
		break;
		
		case off :
			PORTA &= ~BIT0;				//DDC out = 0
		break;
	}
}

void load_pulses(void)
{
	switch (shutter_model)
	{
		case 1 :	//nvidia
			switch (out_sync)
			{
				case right :
					//right on
					ir_tx[0]= nv_ron_1h-6;		//pulse1 ON state
					ir_tx[1]= nv_ron_1l-6;		//pulse1 OFF state
					ir_tx[2]= nv_ron_2h-6;		//pulse2 ON state
					//dark_time
					ir_tx[3]= darktime-6;
					//left off
					ir_tx[4]= nv_loff_1h-6;		//pulse1 ON state
					
					nbr_pulses= (nv_nbrpuls_r*2)-1;
				break;
				
				case left :
					//lefton
					ir_tx[0]= nv_lon_1h-6;		//pulse1 ON state
					ir_tx[1]= nv_lon_1l-6;		//pulse1 OFF state
					ir_tx[2]= nv_lon_2h-6;		//pulse2 ON state
					//dark time
					ir_tx[3]= darktime-6 + (nv_ron_1h+nv_ron_1l+nv_ron_2h)-(nv_lon_1h+nv_lon_1l+nv_lon_2h);
					//right off
					ir_tx[4]= nv_roff_1h-6;		//pulse1 ON state
					ir_tx[5]= nv_roff_1l-6;		//pulse1 OFF state
					ir_tx[6]= nv_roff_2h-6;		//pulse2 ON state
					
					nbr_pulses= (nv_nbrpuls_l*2)-1;
				break;
				
				case off :
					//shutters off
					ir_tx[0]= nv_off_1h-6;		//pulse1 ON state
					ir_tx[1]= nv_off_1l-6;		//pulse1 OFF state
					ir_tx[2]= nv_off_2h-6;		//pulse2 ON state
					nbr_pulses= (nv_nbrpuls_off*2)-1;
				break;
				
				default:
				break;
			}
		break;
		
		case  2 :		//Xpand
			switch (out_sync)
			{
				case right :
					//timings left
					ir_tx[0]= xp_0h-6;	//pulse1 ON state
					ir_tx[1]= xp_0l-6;	//pulse1 OFF state
					ir_tx[2]= xp_0h-6;	//pulse2 ON state
					nbr_pulses= 3;
				break;
				
				case left :
					//timings right
					ir_tx[0]= xp_1h-6;	//pulse1 ON state
					ir_tx[1]= xp_1l-6;	//pulse1 OFF state
					ir_tx[2]= xp_1h-6;	//pulse2 ON state
					ir_tx[3]= xp_1l-6;	//pulse2 OFF state
					ir_tx[4]= xp_1h-6;	//pulse3 ON state
					nbr_pulses= 5;
					
					if(send_code)
					{
						send_code = 0;
						ir_tx[5]= 200;		//delay between command and code
						no_bit=6;
						
						for(i=0;i<16;i++)
						{
							if( ((xp_code >> (15-i))&BIT0) == 0) // Codebit=0
							{
								ir_tx[no_bit]	= xp_0h-6;	//pulse1 ON state
								ir_tx[no_bit+1]	= xp_0l-6;	//pulse1 OFF state
								no_bit += 2;
							}
							else							// Codebit=1
							{
								ir_tx[no_bit] 	= xp_1h-6;	//pulse1 ON state
								ir_tx[no_bit+1]	= xp_1l-6;	//pulse1 OFF state
								ir_tx[no_bit+2]	= xp_1h-6;	//pulse2 ON state
								ir_tx[no_bit+3]	= xp_1l-6;	//pulse2 OFF state
								no_bit += 4;
							}
							ir_tx[no_bit] = xp_0h-6;		//stop bit
						}
						nbr_pulses = no_bit + 1;
					}
				break;
				
				default:
				break;
			}
		break;
		
		case  3 :		//Crystaleyes
			switch (out_sync)
			{
				case right :
					ir_tx[0]= cr_rh-6;
					nbr_pulses= cr_nbrpuls_r;
				break;
				
				case left :
					ir_tx[0]= cr_lh-6;
					nbr_pulses= cr_nbrpuls_l;
				break;
			}
		break;
	}
}

//Load default parameters
void set_parameters(void)
{
	switch (mode)
	{
		case ir_off :
			cons_delay =			ir_off_delay;
			cons_invert =			ir_off_invert;
			ddc_enable =			1;
			ir_enable =				0;
			cons_color_correct =	1;
			shutter_model =			0;
		break;
		
		case nvidia :
			cons_delay =			nv_delay;
			cons_invert =			nv_invert;
			cons_darktime =			nv_darktime;
			darktime =				nv_darktime*2;
			ddc_enable =			0;
			ir_enable =				1;
			cons_color_correct =	0;
			shutter_model =			1;
			load_pulses();
		break;
		
		case nvidia_ddc :
			cons_delay =			nv_ddc_delay;
			cons_invert =			nv_ddc_invert;
			cons_darktime =			nv_ddc_darktime;
			darktime =				nv_ddc_darktime*2;
			ir_ddc_delay =			nv_ddc_ir_ddc_delay;
			ddc_enable =			1;
			ir_enable =				1;
			cons_color_correct =	1;
			shutter_model=			1;
			load_pulses();
		break;
		
		case xpand :
			cons_delay =			xp_delay;
			cons_invert =			xp_invert;
			ddc_enable =			0;
			ir_enable =				1;
			cons_color_correct =	1;
			shutter_model =			2;
			load_pulses();
		break;
		
		case crystaleyes :
			cons_delay =			cr_delay;
			cons_invert =			cr_invert;
			ddc_enable =			0;
			ir_enable =				1;
			cons_color_correct =	1;
			shutter_model =			3;
			load_pulses();
		break;
	}	
}

//Gestion du bouton rotatif
unsigned int aquis_rot(unsigned int nbr, unsigned int inc, unsigned int min, unsigned int max)
{
	//Encoder management (rot1 = sync, rot2 = +/-)
	if(buff_rot1 == 0 && rot1 == 1)
	{	
		if(rot2)
		{
			if(nbr > (min + inc))
				nbr = nbr-inc;
			else
				nbr = min;
		}
		else 
		{
			if(nbr < (max-inc))
				nbr = nbr+inc;
			else
				nbr = max;
		}
	}
	
	if(buff_rot1 == 1 && rot1 == 0)
	{	
		if(rot2)
		{
			if(nbr < (max-inc))
				nbr = nbr+inc;
			else
				nbr = max;
		}
		else 
		{
			if(nbr > (min + inc))
				nbr = nbr-inc;
			else
				nbr = min;
		}
	}
	buff_rot1 = rot1;
	buff_rot2 = rot2;
	return nbr;
}

//push encoder
char push_rot(void)
{
	unsigned char flag_push_rot = 0;
	
	if(buff_bt_rot == 1 && bt_rot == 0)	//falling edge on encoder push input
		flag_push_rot = 1;
	
	buff_bt_rot = bt_rot;
	return flag_push_rot;
}

//button 1
char push_bt1(void)
{
	unsigned char flag_bt1 = 0;
	
	if(buff_bt1 == 1 && bt1 == 0)	//falling edge on button 1 input
		flag_bt1 = 1;
		
	buff_bt1 = bt1;
	return flag_bt1;
}

//button 2
char push_bt2(void)
{
	unsigned char flag_bt2 = 0;
	
	if(buff_bt2 == 1 && bt2 == 0)	//falling edge on button 2 input
		flag_bt2 = 1;
		
	buff_bt2 = bt2;
	return flag_bt2;
}

//Send char on serial out
static int uart_putchar(char c)
{
    loop_until_bit_is_set(UCSRA, UDRE);
    UDR = c;
    return 0;
}

//Write char on LCD
static int lcd_putchar(char c, FILE *stream) 
{ 
   lcd_putc(c); 
   return 0; 
}
